! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: epfile_mod
!
! Authors:  Michael J. Freyberg - MPE Garching - Germany
!          
! Description: Library of EPIC pn specific routines developed at MPE
!              here: specific to files and filenames 
!                    or to columns in data files
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module epfile_mod

  use dal

  implicit none

  private

  public :: stripSlash, get_io_unit, intersect_dp

contains


! ===============================================
! Strip redundant characters from input filename
! ===============================================
subroutine stripSlash(infilename)

  implicit none
  character(len=*)   ,intent(inout) :: infilename
  integer                           :: nl,il,jl

  nl = len(trim(infilename))
  do il = nl, 1, -1
    if (infilename(il:il) == '/') then
      do jl = il-1, 1, -1
        if (infilename(jl:jl) == '/') then
          infilename(jl:nl) = infilename(jl+1:nl)//' '
        else
          exit
        endif
      enddo
    endif
  enddo
  nl = len(trim(infilename))
  do il = nl, 3, -1
    if (infilename(il-2:il) == '/./') then
      infilename(il-2:nl) = infilename(il:nl)//'  '
    endif
  enddo
  if (infilename(1:2) == './') then
    infilename(1:nl) = infilename(3:nl)//'  '
  endif
! print*,'|',trim(infilename),'|'

end subroutine stripSlash


! ==========================================================================
! suggested by Clive Page in SSC-LUX-TN-0005 (Issue 1.0, 1996 Sept 2)
! ==========================================================================
subroutine get_io_unit(iounit)

  implicit none
  integer, intent(out) :: iounit
  logical              :: lopen, lexist
  do iounit = 99,0,-1
    inquire(unit=iounit, exist=lexist, opened=lopen)
    if (lexist .and. .not. lopen) exit
  enddo
  
end subroutine get_io_unit



      subroutine intersect_dp(a1_b,a1_e,n1,a2_b,a2_e,n2, & 
                              aint_b,aint_e,nintval,ib,ie,n)
! ----------------------------------------------------------------------
!     Version 1.0 03-jul-1991, 2.0 05-dec-2003
! ----------------------------------------------------------------------
!     find the intersection of arrays containing the start and end times
!     of (e.g. good time) intervals, all time variables are DOUBLE
! ----------------------------------------------------------------------
!     A#_B     start times of good periods (#=1..2)
!     A#_E     end times of good periods (#=1..2)
!     N#       number of times in time array (#=1..2)
!     NINT     upper limit for intersection times
!     IB       start time for intersection analysis
!     IE       end time for intersection analysis
! ----------------------------------------------------------------------
!     AINT_B   start times of good periods (intersection)
!     AINT_E   end times of good periods (intersection)
!     N        number of times found with IB < time < IE
! ----------------------------------------------------------------------
!     Author: MPECR1::MJF (mjf@mpe.mpg.de)
! ----------------------------------------------------------------------

      implicit none
      integer, intent(in)           :: n1,n2
      double precision, intent(in)  :: a1_b(n1),a1_e(n1)
      double precision, intent(in)  :: a2_b(n2),a2_e(n2)
      double precision, intent(in)  :: ib,ie
      integer,intent(inout)         :: nintval
      integer,intent(out)           :: n
      double precision, intent(out) :: aint_b(nintval),aint_e(nintval)
      integer                       :: l1,l2,m1,m2
      double precision              :: tb1,tb2,te1,te2,tb,te

      l1 = 1
      do while (a1_b(l1).lt.ib)
         l1 = l1+1
      enddo
      l2 = 1
      do while (a2_b(l2).lt.ib)
         l2 = l2+1
      enddo
!
!      write(52,'(1x,a,i11,3i4)') 'start time is:',ib,l1,l2
!
      m1 = l1
      do while (m1.le.n1.and.a1_e(m1).lt.ie)
         m1 = m1+1
      enddo
      m2 = l2
      do while (m2.le.n2.and.a2_e(m2).lt.ie)
         m2 = m2+1
      enddo
!
!      write(52,'(1x,a,i11,3i4)') 'end time is  :',ie,m1,m2
!
!  now:   IB  <  A#_B(L#)  <  IE  <  A#_E(M#)
!
      n = 0
      do while ((l1.le.m1).and.(l2.le.m2))
         tb1 = a1_b(l1)
         te1 = a1_e(l1)
         tb2 = a2_b(l2)
         te2 = a2_e(l2)
         tb = max(tb1,tb2)
         te = min(te1,te2)
         if (te.gt.tb) then
            n = n + 1
            aint_b(n) = tb 
            aint_e(n) = te
         endif
!         write(52,'(1x,3(i4,2i9,1x),2i10,i4,2i9)') l1,a1_b(l1),a1_e(l1),
!     &     l2,a2_b(l2),a2_e(l2),l3,a3_b(l3),a3_e(l3),tb,te,
!     &     n,aint_b(n),aint_e(n)
         if (te1.eq.te) l1 = l1 + 1
         if (te2.eq.te) l2 = l2 + 1
      enddo

!      do l1=1,n
!         write(52,'(1x,i4,2i10)') l1,aint_b(l1),aint_e(l1)
!      enddo

      end subroutine intersect_dp



end module epfile_mod
