! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!+
! Module: eputils_mod
!
! Authors:  Michael J. Freyberg - MPE Garching - Germany
!          
! Description: Library of EPIC pn specific routines developed at MPE
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module epprintinfo_mod

  use caloaldefs
  use dal
  use cal
  use oal
  use errorHandling

  implicit none

  private 

  public :: printODFinfo, printPROPOSALinfo, printEXPOSUREinfo
  public :: SourcePositionOnDetector

contains 


! =============================================================================
! print ODF infos
! =============================================================================
  subroutine printODFinfo(odfinfos)

  type(OdfInfoType), intent(out)   :: odfinfos 

  call message(AppMsg,Noisy+1,"... entering OAL_odfInfo...")
  call OAL_odfInfo(odfinfos)

  if (verbosity(AppMsg) >= Verbose) then 
    write(errstr,*) "XMM ODF in directory  : ",trim(odfinfos%directoryName)
    call message(AppMsg,Verbose)  
    write(errstr,*) "revolution number     :",odfinfos%revolutionNr    
    call message(AppMsg,Verbose)
    write(errstr,*) "proposal id           : ",odfinfos%proposalId
    call message(AppMsg,Verbose)
    write(errstr,*) "total number of files :",odfinfos%nrFiles
    call message(AppMsg,Verbose)
    write(errstr,*) "observation id        : ",odfinfos%observationId
    call message(AppMsg,Verbose)
    write(errstr,*) "observation start time: ", &
                 odfinfos%observationStartTime
    call message(AppMsg,Verbose)
    write(errstr,*) "observation end time  : ", &
                 odfinfos%observationEndTime
    if (odfInfos%isSlew) then
      write(errstr,*) "type of ODF           : SLEW"
    else
      write(errstr,*) "type of ODF           : POINTING"  
    endif
    call message(AppMsg,Verbose)
  endif

  end subroutine printODFinfo 



! =============================================================================
! print PROPOSAL infos
! =============================================================================
  subroutine printPROPOSALinfo(proposalinfo)

  type(ProposalInfoType), intent(out)  :: proposalinfo

  call message(AppMsg,Noisy+1,"... entering OAL_proposalInfo...")
  call OAL_proposalInfo(proposalinfo)

  if (verbosity(AppMsg) >= Verbose) then 
    write(errstr,*) "name of the GO        : ",trim(proposalinfo%GoName)
    call message(AppMsg,Verbose)
    write(errstr,*) "institute of the GO   : ",trim(proposalinfo%GoInstitute)
    call message(AppMsg,Verbose)
    write(errstr,*) "postal addr. of the GO: ",trim(proposalinfo%GoAddress)
    call message(AppMsg,Verbose)
    write(errstr,*) "e-mail addr. of the GO: ",trim(proposalinfo%GoEmail)
    call message(AppMsg,Verbose)
    call message(AppMsg,Verbose,"")
    write(errstr,*) "AO of proposal        : ",trim(proposalinfo%AO)
    call message(AppMsg,Verbose)
    write(errstr,*) "science type          : ",trim(proposalinfo%scienceType)
    call message(AppMsg,Verbose)  
    write(errstr,*) "name of the target    : ",trim(proposalinfo%targetName)
    call message(AppMsg,Verbose)
    call message(AppMsg,Verbose,"")
    write(errstr,"(a,f10.5)") " Ra of target     [deg]: ",proposalinfo%targetRa
    call message(AppMsg,Verbose)
    write(errstr,"(a,f10.5)") " Dec of target    [deg]: ",proposalinfo%targetDec
    call message(AppMsg,Verbose)
    write(errstr,"(a,f10.5)") " Ra of boresight  [deg]: ",proposalinfo%boresightRa
    call message(AppMsg,Verbose)
    write(errstr,"(a,f10.5)") " Dec of boresight [deg]: ",proposalinfo%boresightDec
    call message(AppMsg,Verbose)
    write(errstr,"(a,f10.1)") " duration of obser. [s]: ",proposalinfo%obsDuration
    call message(AppMsg,Verbose)
    call message(AppMsg,Verbose,"")
    write(errstr,*) "pos. angle constraints: ", &
          trim(proposalinfo%posAngleConstLo), trim(proposalinfo%posAngleConstHi)
    call message(AppMsg,Verbose)
    write(errstr,*) "pos. angle orig. ref. : ", &
          trim(proposalinfo%posAngleOrigRef)
    call message(AppMsg,Verbose)
    call message(AppMsg,Verbose,"")
    write(errstr,*) "instrument priorities : emos1: ",proposalinfo%emos1Prio
    call message(AppMsg,Verbose)
    write(errstr,*) "                        emos2: ",proposalinfo%emos2Prio
    call message(AppMsg,Verbose)
    write(errstr,*) "                        epn  : ",proposalinfo%epnPrio
    call message(AppMsg,Verbose)
    write(errstr,*) "                        rgs1 : ",proposalinfo%rgs1Prio
    call message(AppMsg,Verbose)
    write(errstr,*) "                        rgs2 : ",proposalinfo%rgs2Prio
    call message(AppMsg,Verbose)
    write(errstr,*) "                        om   : ",proposalinfo%omPrio
    call message(AppMsg,Verbose)    
    call message(AppMsg,Verbose,"")
  endif

  end subroutine printPROPOSALinfo




! =============================================================================
! print EXPOSURE infos
! =============================================================================
  subroutine printEXPOSUREinfo(expinfos,nCCDs,filter)

  integer(kind=Int16), intent(in)         :: nCCDs
  character(len=*), intent(in)            :: filter
  type(ExposureInfoType), intent(out)     :: expinfos

  call message(AppMsg,Noisy+1,"... entering OAL_exposureInfo...")
  call OAL_exposureInfo(expInfos)

  if (verbosity(AppMsg) >= Noisy) then
    write(errstr,"(a,i2,a,i2,a)") " exposure number       : ", &
          expInfos%number," (with ",nCCDs," active CCDs)"
    call message(AppMsg,Verbose)
    write(errstr,*)  "exposure scheduled?   :", expInfos%scheduled
    call message(AppMsg,Noisy)
    if (expInfos%type == 0) then
      call message(AppMsg,Noisy," type of exposure      : SCIENCE")
    else
      call message(AppMsg,Noisy," type of exposure      : CALIBRATION")
    endif
    write(errstr,*) "active filter         : ", filter
    call message(AppMsg,Noisy)
    write(errstr,*) "scheduled start time  : ", &
          expInfos%scheduledStartTime(1:23)
    call message(AppMsg,Noisy)
    write(errstr,*) "actual    start time  : ", &
          expInfos%actualStartTime(1:23)
    call message(AppMsg,Noisy)
    write(errstr,*) "scheduled end   time  : ", &
          expInfos%scheduledEndTime(1:23)
    call message(AppMsg,Noisy)
    write(errstr,*) "actual    end   time  : ", &
          expInfos%actualEndTime(1:23)
    call message(AppMsg,Noisy)
    write(errstr,*) "scheduled length of exposure [s]: ", &
          toTimeTag(expInfos%scheduledEndTime(1:23)) - &
          toTimeTag(expInfos%scheduledStartTime(1:23))
    call message(AppMsg,Noisy)
    write(errstr,*) "actual    length of exposure [s]: ", &
          toTimeTag(expInfos%actualEndTime(1:23)) - &
          toTimeTag(expInfos%actualStartTime(1:23))
    call message(AppMsg,Noisy)
    call message(AppMsg,Noisy,"")
  endif

  end subroutine printEXPOSUREinfo 


! ================================================================================
  subroutine SourcePositionOnDetector(sourceDir,AttTimeTag,xrawsrc,yrawsrc,ccdsrc)

  implicit none
  type(EquatorialDirectionType), dimension(0:0), intent(in) :: sourceDir
  real(kind=double),             intent(in)  :: AttTimeTag
  real(kind=double),             intent(out) :: xrawsrc, yrawsrc
  integer,                       intent(out) :: ccdsrc

  type(SpacecraftAttitudeType)               :: scAtt
  type(BoresightT)                           :: boresight
  real(kind=double), dimension(:), pointer   :: theta => null(),phi => null(),&
                                                x => null(),y => null(),z => null()
  real(kind=double), dimension(:), pointer   :: RchipX => null(), RchipY => null()
  integer(kind=int16), dimension(:), pointer :: chipX => null(), chipY => null()
  integer(kind=int8) , dimension(:), pointer :: ccds => null()
  integer :: i
  xrawsrc = 0.0d0
  yrawsrc = 0.0d0

  if ( OAL_getAttitude(AttTimeTag, scAtt) ) then
    boresight = CAL_getBoresight(AttTimeTag)

    call CAL_withSacCoordOrientation(boresight, scAtt)
    if (verbosity(AppMsg) >= Noisy) then
      call message(AppMsg,Noisy,' ')
      write(errstr,*) 'AttTimeTag,SCATT = ',AttTimeTag,scAtt
      call message(AppMsg,Noisy)
    endif

    call CAL_projectOntoTelCoord(boresight, sourceDir, theta, phi)
    if (verbosity(AppMsg) >= Noisy) then
      write(errstr,*) 'SOURCEDIR =',(sourceDir(i), i=0,size(sourceDir)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'THETA =',(theta(i), i=0,size(theta)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'PHI =',(phi(i), i=0,size(phi)-1)
      call message(AppMsg,Noisy)
    endif

    call CAL_telCoordToCamCoord2(theta, phi, x, y, z)
    if (verbosity(AppMsg) >= Noisy) then
      write(errstr,*) 'THETA = ',(theta(i), i=0,size(theta)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'PHI = ', (phi(i), i=0,size(phi)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'X = ', (x(i), i=0,size(x)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'Y = ',(y(i), i=0,size(y)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'Z = ',(z(i), i=0,size(z)-1)
      call message(AppMsg,Noisy)


    endif

    call CAL_camCoord2ToCamCoord1(x, y, z, x, y, z)
    if (verbosity(AppMsg) >= Noisy) then
      write(errstr,*) 'X = ', (x(i), i=0,size(x)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'Y = ',(y(i), i=0,size(y)-1)
      call message(AppMsg,Noisy)
      write(errstr,*) 'Z = ',(z(i), i=0,size(z)-1)
      call message(AppMsg,Noisy)
    endif

    call CAL_camCoord1ToChipCoord(x, y, z, chipX, chipY, ccds)
    if (verbosity(AppMsg) >= Noisy) then
      write(errstr,*) 'chipX = ', (chipX(i), i=0,size(chipX)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
      write(errstr,*) 'chipY = ',(chipY(i), i=0,size(chipY)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
      write(errstr,*) 'ccds = ',(ccds(i), i=0,size(ccds)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
    endif
    xrawsrc = chipX(0)
    yrawsrc = chipY(0)
    ccdsrc = ccds(0)

    call CAL_camCoord1ToChipRealCoord(x, y, z, RchipX, RchipY, ccds)
    if (verbosity(AppMsg) >= Noisy) then
      write(errstr,*) 'RchipX = ', (RchipX(i), i=0,size(RchipX)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
      write(errstr,*) 'RchipY = ',(RchipY(i), i=0,size(RchipY)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
      write(errstr,*) 'ccd = ',(ccds(i), i=0,size(ccds)-1)
      call message(AppMsg,Noisy)
      call message(AppMsg,Noisy,' ')
    endif
    xrawsrc = RchipX(0)
    yrawsrc = RchipY(0)
    ccdsrc = ccds(0)

    call CAL_releaseMemory(theta, phi)
    call CAL_releaseMemory(x, y, z)
    call CAL_releaseMemory(chipX, chipY)
    call CAL_releaseMemory(RchipX, RchipY)
    call CAL_releaseMemory(ccds)
  endif

  end subroutine SourcePositionOnDetector



end module epprintinfo_mod
